/*
 * Copyright 2012, 2013 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

#include "com_redhat_thermostat_agent_proxy_server_AgentProxyNativeUtils.h"

#include <unistd.h>
#include <string.h>
#include <errno.h>

static jint throw_Exception(JNIEnv *, const char *, const char *);

JNIEXPORT void JNICALL 
Java_com_redhat_thermostat_agent_proxy_server_AgentProxyNativeUtils_setCredentials(
        JNIEnv *env, jclass splitAgentKlass, jlong uid, jlong gid) {
    int rc;
    char *err;

    // Need to setegid before seteuid, or otherwise we've dropped permissions
    // needed to do so.
    rc = setegid(gid);
    if (rc < 0) {
        err = strerror(errno);
        throw_Exception(env, "java/lang/Exception", err);
    }

    rc = seteuid(uid);
    if (rc < 0) {
        err = strerror(errno);
        throw_Exception(env, "java/lang/Exception", err);
    }
}

static jint throw_Exception(JNIEnv *env, const char *class_name,
        const char *message) {
    jclass class;

    class = (*env)->FindClass(env, class_name);
    if (class == NULL) {
        return -1;
    }
    return (*env)->ThrowNew(env, class, message);
}
